/**
  * Copyright (c) 2000-2004 Jim Holmstrm. All rights reserved.
  *
  * The author grants you the right to use, modify, and redistribute this
  * software for noncommercial purposes, on the conditions that you:
  * (1) retain the original headers, including the copyright notice and
  * this text, (2) clearly document the difference between any derived
  * software and the original, and (3) acknowledge your use of this
  * software in pertaining publications and reports.  This software
  * is provided "as is", without any warranty of any kind.
  *
  * Author  : Jim Holmstrm
  * E-mail  : jimh@it.uu.se
  *           boll@booru.net
  * Web     : http://user.it.uu.se/~jimh
  *           http://www.booru.net
  *
  */

#ifndef __CONTROL_H
#define __CONTROL_H

#include <windows.h> 

/**
 * This is the abstract base class for Control Objects, it should be 
 * inherited to define a specific behaviour for some implementation. 
 * The Iterate method should be called to perform an iteration.
 * If iterate returns false, that means it is time to stop and no more iterations
 * will be performed.
 */
class Control
{
public:
	virtual ~Control() { }

    /**
     * Sets the delay in milliseconds between each iteration.
     * A call to iterate will sleep until it is time to perform that iteration.
     * @param d the delay in milliseconds.
     */
	virtual void SetDelay(int d) 
	{ 
		m_delay = d; 
		m_lastTime -= m_delay;
	}
	
    /**
     * Returns the current iteration count.
     * @return the current iteration count.
     */
	virtual unsigned int GetIteration() { return m_iteration; }	

	/**
	* The main iteration loop.
	* @return false when iteration should be ceased, true if continue.
	*/
	virtual bool Iterate() = 0;
	
	/**
    * A method provided as to allow saving states to disc and so forth.
    */
    virtual void Snapshot() {}
	
protected:
	Control()
	{
		m_lastTime = GetTickCount();
		m_iteration = 0;
	}
	
	virtual bool IsReadyToIterate() 
	{ 
		if(m_delay==0)
			return true;
		
		unsigned int currentTime = GetTickCount();
		if(currentTime > m_lastTime + m_delay)
		{
			m_lastTime = currentTime;
			return true;
		}
		return false;
	}
	
	virtual bool IsStopCriteriaMet() = 0;
	
protected:
	int             m_delay;
	unsigned int    m_lastTime;
	unsigned int    m_iteration;	
};


#endif
