/**
  * Copyright (c) 2000-2004 Jim Holmstrm. All rights reserved.
  *
  * The author grants you the right to use, modify, and redistribute this
  * software for noncommercial purposes, on the conditions that you:
  * (1) retain the original headers, including the copyright notice and
  * this text, (2) clearly document the difference between any derived
  * software and the original, and (3) acknowledge your use of this
  * software in pertaining publications and reports.  This software
  * is provided "as is", without any warranty of any kind.
  *
  * Author  : Jim Holmstrm
  * E-mail  : jimh@it.uu.se
  *           boll@booru.net
  * Web     : http://user.it.uu.se/~jimh
  *           http://www.booru.net
  *
  */

//-----------------------------------------------------------------------------
// Author: Jim Holmstrm
// Date:   2002-02-13
//-----------------------------------------------------------------------------

#include <Vector.hpp>

#include <GNGCore/IGNGContainer.hpp>
#include <GNGCore/GNGNode.hpp>
#include <GNGCore/GNGEdge.hpp>

#include "DefaultNodeInserter.hpp"
#include <iostream>

using namespace std;


//----------
DefaultNodeInserter::DefaultNodeInserter()
{
  m_container = 0;
  m_iteration = 0;
}

//----------
void DefaultNodeInserter::SetParamInsertEvery(unsigned int i) 
{ 
  m_paramInsertEvery = i; 
}

//----------
void DefaultNodeInserter::SetParamMaxNodes(unsigned int i)    
{ 
  m_paramMaxNodes = i; 
}

//----------
void DefaultNodeInserter::SetParamNewNodePos(double i)        
{ 
  if(i < 0.0)
    i = 0.0;
  if(i > 1.0)
    i = 1.0;
    
  m_paramNewNodePos = i;  
}

//----------
void DefaultNodeInserter::SetParamErrorDecay(double i)        
{ 
  if(i < 0.0)
    i = 0.0;
  if(i > 1.0)
    i = 1.0;
    
  m_paramErrorDecay = i; 
}



//----------
bool DefaultNodeInserter::IsInsertCriteriaMet()
{
  if(m_container->GetNodes().size() >= m_paramMaxNodes)
    return false;

  m_iteration++;
  if(m_iteration >= m_paramInsertEvery)
  {
    m_iteration = 0;
    return true;
  }

  return false;
}






//----------
GNGNode * DefaultNodeInserter::FindNodeWithLargestError()
{
  NodeContainer const & nodes = m_container->GetNodes();
  NodeContainer::const_iterator it = nodes.begin();

  if(it == nodes.end())
    throw Exception("FindeNodeWithLargestError:: There are NO nodes!");

  GNGNode * node = *it;
  it++;

  double maxError = node->GetAccumulatedError();

  for(; it != nodes.end(); it++)
  {
    GNGNode const & currentNode = *(*it);
    double error = currentNode.GetAccumulatedError();

    if(error > maxError)
    {
      node = *it;
      maxError = error;
    }
  }

  return node;
}

//----------
GNGNode * DefaultNodeInserter::FindNeighborWithLargestError(GNGNode const & u)
{
  EdgeContainer const & edges = u.GetEdges();
  EdgeContainer::const_iterator it = edges.begin();
  
  if(it == edges.end())
    throw Exception("FindNeighborWithLargestError:: There are NO edges!");
  
  GNGNode * node = &(*it)->GetOtherNode(u);
  it++;

  double maxError = node->GetAccumulatedError();

  for(; it != edges.end(); it++)
  {
    GNGNode & currentNode = (*it)->GetOtherNode(u);
    double error = currentNode.GetAccumulatedError();

    if(error > maxError)
    {
      node = &currentNode;
      maxError = error;
    }
  }

  return node;
}


//----------
void DefaultNodeInserter::InsertNewNode()
{
  try 
	{
		// step 1
		GNGNode * nodeU = FindNodeWithLargestError();
		
		// step 2
		GNGNode * nodeV = FindNeighborWithLargestError(*nodeU);
		
		// step 3
		Vector & refU = nodeU->GetReferenceVector();
		Vector & refV = nodeV->GetReferenceVector();
		
		GNGNode * nodeR = m_container->CreateNode(); // CreateNode stores the node in the GNGContainer.
		nodeR->SetReferenceVector( refU + ((refV - refU) * m_paramNewNodePos) );
		
		// step 4
		GNGEdge * edge1 = m_container->CreateEdge(nodeU, nodeR); // CreateEdge stores the edge in the GNGContainer. 
		GNGEdge * edge2 = m_container->CreateEdge(nodeV, nodeR);
		
		edge1->SetAge(0);
		edge2->SetAge(0);
		
		m_container->DeleteEdge( nodeU->GetEdgeTo(*nodeV) );
		
		
		// according to the article on GNG by fritzke
		double errorU = nodeU->GetAccumulatedError() * m_paramErrorDecay;
		double errorV = nodeV->GetAccumulatedError() * m_paramErrorDecay;
		double errorR = NewError(errorU, errorV);
			
		double utilityR = (nodeU->GetUtility() + nodeV->GetUtility()) / 2.0;
		
		nodeU->SetAccumulatedError(errorU);
		nodeV->SetAccumulatedError(errorV);
		nodeR->SetAccumulatedError(errorR);
		nodeR->SetUtility(utilityR);
		
  }
  catch(Exception e)
  {
    cout << "InsertNewNode:: Node not inserted!" << endl;
  }

}

double DefaultNodeInserter::NewError(double errorU, double errorV)
{
    return (errorU + errorV) / 2.0; // according to Fritzkes java implementation
    //return errorU; // according to the article.
}

//----------
void DefaultNodeInserter::SetGNGContainer(IGNGContainer * c)
{
  m_container = c;
}



