/**
  * Copyright (c) 2000-2004 Jim Holmstrm. All rights reserved.
  *
  * The author grants you the right to use, modify, and redistribute this
  * software for noncommercial purposes, on the conditions that you:
  * (1) retain the original headers, including the copyright notice and
  * this text, (2) clearly document the difference between any derived
  * software and the original, and (3) acknowledge your use of this
  * software in pertaining publications and reports.  This software
  * is provided "as is", without any warranty of any kind.
  *
  * Author  : Jim Holmstrm
  * E-mail  : jimh@it.uu.se
  *           boll@booru.net
  * Web     : http://user.it.uu.se/~jimh
  *           http://www.booru.net
  *
  */

//-----------------------------------------------------------------------------
// Author: Jim Holmstrm
// Date:   2002-02-13
//-----------------------------------------------------------------------------

#ifndef __DEFAULTNODEINSERTER_H
#define __DEFAULTNODEINSERTER_H

#include "../GNGCore/IGNGNodeInserter.hpp"
class IGNGContainer;
class GNGNode;

/**
 * The default node inserter, inserts nodes at fixed intervalls.
 */
class DefaultNodeInserter : public IGNGNodeInserter
{
public:
  DefaultNodeInserter();

  /**
   * Sets the IGNGContainer component of DefaultNodeInserter.
   * It is used to create edges and nodes. It is the actual 
   * container of the nodes and edges.
   * @param c the IGNGContainer component.
   */
  virtual void SetGNGContainer(IGNGContainer * c);

  /**
   * Checks if the insert new node criteria is met.
   * @return true if criteria is met false otherwise.
   */
  virtual bool IsInsertCriteriaMet();

  /**
   * Inserts a new Node, at the apropriate postition.
   * The IGNGContainer component managed nodes and edges
   * and is used to create them.
   */
  virtual void InsertNewNode();

  /**
   * Sets the InsertEvery parameter. It controls
   * the frequency of insertions.
   * @param i the parameter value.
   */
  void SetParamInsertEvery(unsigned int i);

  /**
   * Sets the MaxNodes parameter. It controls
   * how many nodes are allowed to be created.
   * @param i the parameter value.
   */
  void SetParamMaxNodes(unsigned int i);

  /**
   * Sets the NewNodePos parameter. It controls 
   * the interpolation between the two most erroneous nodes U and V. 
   * It should be a value between 0.0 and 1.0.
   * 1.0 means the new node will be placed as far away from the most
   * erroneous node U as possible, that is on the same spot as V.
   * 0.0 means the new node will be placed in the same spot as U.
   * @param i the interpolation parameter.
   */
  void SetParamNewNodePos(double i);

  /**
   * Sets the ErrorDecay parameter. It scales the local error of 
   * the nodes U and V with the hightst error after a new node
   * has been inserted between U and V. It should be a value between
   * 0.0 and 1.0. 1.0 leaves the error intact and 0.0 makes the error 0.0.
   */
  void SetParamErrorDecay(double i);


protected:

  /**
   * Finds the Node U with the largest local error.
   * @return the node U with the largest local error.
   */
  virtual GNGNode * FindNodeWithLargestError();

  /**
   * Finds the Neighbor V of node U with the largest error.
   * @return the node V.
   */
  virtual GNGNode * FindNeighborWithLargestError(GNGNode const & u);

  /**
   * Defines how the new error is initialized for the new node.
   * @param errorU the error of the winner-node
   * @param errorV the error of the second closest node
   * @return the new error
   */
  virtual double NewError(double errorU, double errorV);
  
protected:
  unsigned int    m_iteration;

  unsigned int    m_paramInsertEvery;
  unsigned int    m_paramMaxNodes;
  double          m_paramNewNodePos;
  double          m_paramErrorDecay;

  IGNGContainer * m_container;
};

#endif

