/**
  * Copyright (c) 2000-2004 Jim Holmstrm. All rights reserved.
  *
  * The author grants you the right to use, modify, and redistribute this
  * software for noncommercial purposes, on the conditions that you:
  * (1) retain the original headers, including the copyright notice and
  * this text, (2) clearly document the difference between any derived
  * software and the original, and (3) acknowledge your use of this
  * software in pertaining publications and reports.  This software
  * is provided "as is", without any warranty of any kind.
  *
  * Author  : Jim Holmstrm
  * E-mail  : jimh@it.uu.se
  *           boll@booru.net
  * Web     : http://user.it.uu.se/~jimh
  *           http://www.booru.net
  *
  */

//-----------------------------------------------------------------------------
// Author: Jim Holmstrm
// Date:   2002-02-13
//-----------------------------------------------------------------------------

#include <Random.hpp>
#include <Exception.hpp>
#include <Vector.hpp>

#include "GNGAlgorithm.hpp"
#include "NodeContainer.hpp"
#include "EdgeContainer.hpp"
#include "GNGEdge.hpp"
#include "GNGNode.hpp"
#include "IGNGContainer.hpp"
#include "IGNGNodeInserter.hpp"
#include "IGNGNodeMovement.hpp"

#include <Generators/IInputGenerator.hpp>

#include <cstdlib>

#define DEF_MSE_HISTORY 500
using namespace std;

//-----------
GNGAlgorithm::GNGAlgorithm(unsigned int MSEBackLog)
{
  m_MSEBackLog = MSEBackLog;
  m_dimension = 0;

  m_gngContainer = 0;
  m_nodeInserter = 0;
  m_stopCritera = 0;
  m_winnerMovement = 0;
  m_neighborMovement = 0;

  m_initDone = false;
  m_mseStream = 0;
  m_mse = 0.0;	
}

//-----------
void GNGAlgorithm::Initialize(unsigned int dim, IInputGenerator * g)
{
  m_dimension = dim;
  if(m_dimension == 0)
    throw Exception("GNGAlgorithm:: sorry, don't support 0-dimensional space.");

  m_input = Vector(m_dimension);

  Vector v1(m_dimension);
  Vector v2(m_dimension);

  // if an input generator is provided during initialization
  // initialize using the input generator
  //   else initialize randomly
  if(g != 0)
  {
    v1 = g->GetInput();
    v2 = g->GetInput();
  }
  else
  {
    for(unsigned int i=0; i<m_dimension; i++)
    {
      v1[i] = Random::Randomize();
      v2[i] = Random::Randomize();    
    } 
  }

  // create 2 nodes
  GNGNode * n1 = m_gngContainer->CreateNode();
  GNGNode * n2 = m_gngContainer->CreateNode();

  // create an initial edge between them, this always happens at first anyway.
  m_gngContainer->CreateEdge(n1, n2);


  n1->SetReferenceVector(v1);
  n2->SetReferenceVector(v2);

  m_initDone = true;
}
 

//-----------
void GNGAlgorithm::Iterate(Vector const & input)
{
  if(!m_initDone)
    throw Exception("Iterate::Init must be called first.");

  m_input = input;

  // Step 2
  //   locate two nodes closest to the input.
  GNGNode * s = 0;
  GNGNode * t = 0;
  GetClosestNodes(s, t);

  // Step 3
  UpdateError(*s);

  // Step 4
  //   Increase age of all edges to s's topological neighbors
  IncreaseEdgeAges(*s);
    
  // Step 5
  MoveWinner(*s);
  MoveNeighbors(*s);

  // Step 6
  GNGEdge * edge = s->GetEdgeTo(*t);
  if( edge != 0 )
    edge->SetAge(0);
  else
    m_gngContainer->CreateEdge(s, t);
    
  // Step 7
  PurgeEdges();
  PurgeNodes();

  // Step 8
  if( m_nodeInserter->IsInsertCriteriaMet() )
    m_nodeInserter->InsertNewNode();

  // Step 9
  DecreaseNodeErrors();

}


//----------------------------------------------------------------------------
// 
//----------------------------------------------------------------------------


//-----------
// Finds the two nodes closest to input
void GNGAlgorithm::GetClosestNodes(/*OUT*/GNGNode * & s,/*OUT*/GNGNode * & t)
{  
  NodeContainer & nodes = m_gngContainer->GetNodes();

  if(nodes.size() < 2)
    throw Exception("GetClosestNodes:: There are not enough nodes!");

  NodeContainer::const_iterator it = nodes.begin();

  GNGNode * closestNode = *it;
  it++;
  GNGNode * closestNode2 = *it;
  it++;

  double minDistance = closestNode->GetReferenceVector().SquaredDistance(m_input);
  double minDistance2 = closestNode2->GetReferenceVector().SquaredDistance(m_input);

  if(minDistance2 < minDistance)
  {
    GNGNode * tmp = closestNode;
    closestNode = closestNode2;
    closestNode2 = tmp;

    double tmpDistance = minDistance;
    minDistance = minDistance2;
    minDistance2 = tmpDistance;
  }

  for(; it != nodes.end(); it++)
  {
    GNGNode const & currentNode = *(*it);
    double distance = currentNode.GetReferenceVector().SquaredDistance(m_input);

    if(distance < minDistance)
    {
      closestNode2 = closestNode;
      minDistance2 = minDistance;

      closestNode = *it;
      minDistance = distance;
    }
    else
    if(distance < minDistance2)
    {
      closestNode2 = *it;
      minDistance2 = distance;
    }
  }

  s = closestNode;
  t = closestNode2;

  //-----------------
  // MSE calculation
  //-----------------
  UpdateMSE(minDistance);
}

//-----------
void GNGAlgorithm::UpdateError(GNGNode & s)
{
  double error = s.GetAccumulatedError();
  s.SetAccumulatedError( error + s.GetReferenceVector().SquaredDistance(m_input) );
}

//-----------
// Increases the age of all edges between s and its
// Topological neighbors.
void GNGAlgorithm::IncreaseEdgeAges(GNGNode & s)
{
  EdgeContainer const & edges = s.GetEdges();
  EdgeContainer::const_iterator it = edges.begin();
  for(; it != edges.end(); it++)
    (*it)->SetAge( (*it)->GetAge() + 1.0 );
}


//-----------
void GNGAlgorithm::MoveWinner(GNGNode & s)
{
  Vector const & currentVector = s.GetReferenceVector();
  Vector const & moveVector = m_winnerMovement->GetNewReferenceVector(m_input, currentVector, m_paramMoveWinner);
  s.SetReferenceVector( moveVector );
}

//-----------
void GNGAlgorithm::MoveNeighbor(GNGNode & s)
{
  Vector const & currentVector = s.GetReferenceVector();
  Vector const & moveVector = m_neighborMovement->GetNewReferenceVector(m_input, currentVector, m_paramMoveNeighbor);
  s.SetReferenceVector( moveVector );
}

//-----------
void GNGAlgorithm::MoveNeighbors(GNGNode const & s)
{
  EdgeContainer const & edges = s.GetEdges();
  EdgeContainer::const_iterator it = edges.begin();
  for(; it != edges.end(); it++)
  {
    GNGNode & node = (*it)->GetOtherNode(s);
    MoveNeighbor(node);
  }
}

//-----------
void GNGAlgorithm::PurgeEdges()
{
  EdgeContainer & edges = m_gngContainer->GetEdges();
  EdgeContainer::iterator it = edges.begin();

  while(it != edges.end())
  {
    GNGEdge & edge = *(*it);
    if( edge.GetAge() > m_paramAgeMax )
      m_gngContainer->DeleteEdge(it);
    else
      it++;
  }
  
}

//-----------
void GNGAlgorithm::PurgeNodes()
{
  NodeContainer & nodes = m_gngContainer->GetNodes();
  NodeContainer::iterator it = nodes.begin();

  while(it != nodes.end())
  {
    GNGNode & node = *(*it);
    if( node.GetEdges().empty()  )
      m_gngContainer->DeleteNode(it);
    else
      it++;
  }
}

//-----------
void GNGAlgorithm::DecreaseNodeErrors()
{
  NodeContainer & nodes = m_gngContainer->GetNodes();
  NodeContainer::iterator it = nodes.begin();

  for(;it != nodes.end(); it++)
  {
    GNGNode & node = *(*it);
    double error = node.GetAccumulatedError() * (1.0 - m_paramErrorDecay);
    node.SetAccumulatedError( error );
  }
}

//-----------
void GNGAlgorithm::UpdateMSE(double minDistance)
{
	static double error = 0.0;
	error += minDistance;

	static unsigned int iteration = 0;
	static unsigned int iterationsTotal = 0;
	iteration++;
	iterationsTotal++;

	if(m_mseStream != 0)
		if(iteration >= m_MSEBackLog)
		{
			iteration=0;
			m_mse = error / (double)(m_MSEBackLog); 
			error = 0.0;
			if(m_mseStream->good())
				(*m_mseStream) << iterationsTotal << " " << m_mse << endl;
		}

}




//----------------------------------------------------------------------------
//          COMPONENT and PARAMETER SETUP
//----------------------------------------------------------------------------

//-----------
void GNGAlgorithm::SetComponentGNGContainer(IGNGContainer * i)
{
  m_gngContainer = i;
}

//-----------
void GNGAlgorithm::SetComponentNodeInserter(IGNGNodeInserter * i)
{
  m_nodeInserter = i;
}

//-----------
void GNGAlgorithm::SetComponentWinnerMovement(IGNGNodeMovement * i)
{
  m_winnerMovement = i;
}

//-----------
void GNGAlgorithm::SetComponentNeighborMovement(IGNGNodeMovement * i)
{
  m_neighborMovement = i;
}


//-----------
void GNGAlgorithm::SetParamAgeMax(double d)
{
  m_paramAgeMax = d;
}

//-----------
void GNGAlgorithm::SetParamMoveWinner(double d)
{
  m_paramMoveWinner = d;
}

//-----------
void GNGAlgorithm::SetParamMoveNeighbor(double d)
{
  m_paramMoveNeighbor = d;
}

//-----------
void GNGAlgorithm::SetParamErrorDecay(double d)
{
  m_paramErrorDecay = d;
}



