/**
  * Copyright (c) 2000-2004 Jim Holmstrm. All rights reserved.
  *
  * The author grants you the right to use, modify, and redistribute this
  * software for noncommercial purposes, on the conditions that you:
  * (1) retain the original headers, including the copyright notice and
  * this text, (2) clearly document the difference between any derived
  * software and the original, and (3) acknowledge your use of this
  * software in pertaining publications and reports.  This software
  * is provided "as is", without any warranty of any kind.
  *
  * Author  : Jim Holmstrm
  * E-mail  : jimh@it.uu.se
  *           boll@booru.net
  * Web     : http://user.it.uu.se/~jimh
  *           http://www.booru.net
  *
  */

//-----------------------------------------------------------------------------
// Author: Jim Holmstrm
// Date:   2002-02-13
//-----------------------------------------------------------------------------

#ifndef __GNGALGORITHM_H
#define __GNGALGORITHM_H

#include <Vector.hpp>

#include "IGNGContainer.hpp"
#include "NodeContainer.hpp"
#include "EdgeContainer.hpp"

class GNGEdge;
class GNGNode;
class IGNGNodeInserter;
class IGNGStopCriteria;
class IGNGNodeMovement;
class IInputGenerator;

/**
 *  Encapsulates the GNG algorithm descibed by Fritzke.
 *  Keeps a list of all edges.
 *  Creates and destroys edges and nodes through IGNGContainer.
 *  Updates local accumulated errors and decides where
 *  to insert new nodes using IGNGNodeInserter.
 *  It uses IGNGStopCriteria to decide when to stop.
 */
class GNGAlgorithm
{
public:
  /**
   * Constructs a GNG Algorithm object.
   */
  GNGAlgorithm(unsigned int MSEBackLog);
  virtual ~GNGAlgorithm() 
  {
    m_initDone = true;
  }
  
  /**
   * Initializes the GNG Alg.
   * @param dim the dimension of the inputs and the GNGNodes.
   */ 
  virtual void Initialize(unsigned int dim, IInputGenerator * g = 0);

  /**
   * Sets the output stream used to output the mean squared error.
   * If the mse output stream isn't set, it will not be calculated.
   * @param f, the output stream used for outputing the mse
   */ 
  virtual void SetMSEOutputStream(std::ostream * f) { m_mseStream = f; }
  
  /**
   * Updates the current approximation of the mse, 
   * called once each iteration by iterate.
   * @param minDistance the distance between the input and the closest node.
   */
  virtual void UpdateMSE(double minDistance);

  /**
   * Iterates through the algorithm one time.
   * @param input the Input vector to use in this iteration
   */ 
  virtual void Iterate(Vector const & input);
  
  /**
   * Registers an IGNGContainer with the GNGAlgorithm object.
   * @param i the IGNGContainer to register.
   */
  void SetComponentGNGContainer(IGNGContainer * i);

  /**
   * Registers an IGNGNodeInserter with the GNGAlgorithm object.
   * @param i the IGNGNodeInserter to register.
   */
  void SetComponentNodeInserter(IGNGNodeInserter * i);

  /* *
   * Registers an IGNGStopCriteria with the GNGAlgorithm object.
   * @param i the IGNGStopCriteria to register.
   */
  //void SetComponentStopCriteria(IGNGStopCriteria * i);
  
  
  /**
   * Registers an IGNGNodeMovement with the GNGAlgorithm object.
   * This NodeMovement implementation determines how the winner 
   * node is moved.
   * @param i the IGNGNodeMovement to register.
   */
  void SetComponentWinnerMovement(IGNGNodeMovement * i);

  /**
   * Registers an IGNGNodeMovement with the GNGAlgorithm object.
   * This NodeMovement implementation determines how neighbor nodes
   * of the winner are moved.
   * @param i the IGNGNodeMovement to register.
   */
  void SetComponentNeighborMovement(IGNGNodeMovement * i);

  /**
   * Sets the AgeMax parameter of the GNGAlgorithm
   * @param d the AgeMax parameter.
   */
  void SetParamAgeMax(double d);

  /**
   * Sets the MoveWinner parameter of the GNGAlgorithm
   * @param d the MoveWinner parameter.
   */
  void SetParamMoveWinner(double d);

  /**
   * Sets the MoveNeighbor parameter of the GNGAlgorithm
   * @param d the MoveNeighbor parameter.
   */
  void SetParamMoveNeighbor(double d);

  /**
   * Sets the ErrorDecay parameter of the GNGAlgorithm
   * @param d the ErrorDecay parameter.
   */
  void SetParamErrorDecay(double d);

protected:
  /**
   * Finds the two nodes closest to the current input Vector.
   * @return s and t the closest nodes.
   */
  virtual void GetClosestNodes(/*OUT*/GNGNode * & s, 
			                   /*OUT*/GNGNode * & t);
  /**
   * Updates the local error of a node.
   * @param s the node that will be updated.
   */
  virtual void UpdateError(GNGNode & s);

  /**
   * Increases the age of all edges emenating from the node s.
   * @param s the node connected to the edges.
   */
  virtual void IncreaseEdgeAges(GNGNode & s);

  /**
   * Moves the winner node s towards the current input vector by a factor of paramMoveWinner.
   * To change the behaviour of winner movment, overload this method.
   * @param s the node to move
   */
  virtual void MoveWinner(GNGNode & s);
  
  /**
   * Moves a neighbor node s towards the input vector by a factor of paramMoveNeighbor.
   * To change the behaviour of neighbor movment, overload this method.
   * @param s the node to move
   */
  virtual void MoveNeighbor(GNGNode & s);

  /**
   * Moves the neighbors of the node s towards the input vector.
   * @param s the node whos neigbors are to be moved.
   * @param input the goal of the movement.
   */
  virtual void MoveNeighbors(GNGNode const & s);

  /**
   * Removes edges that are too old.
   */
  virtual void PurgeEdges();

  /**
   * Removes nodes that have no edges.
   */
  virtual void PurgeNodes();

  /**
   * Decreases the local error of all nodes.
   */
  virtual void DecreaseNodeErrors();
  


protected:
  // components
  IGNGNodeInserter *    m_nodeInserter;
  IGNGStopCriteria *    m_stopCritera;
  IGNGNodeMovement *    m_winnerMovement;
  IGNGNodeMovement *    m_neighborMovement;

  IGNGContainer *       m_gngContainer;

  bool                  m_initDone;  
  unsigned int          m_dimension; ///dimension of nodes reference Vectors
  Vector                m_input;

  std::ostream * m_mseStream;  // the stream which to output the mean squared error.
  unsigned int   m_MSEBackLog; // the number of samples to use for the mean sq. error
  double         m_mse;        // keep track of mean squared error m_MSEBackLog number of steps

  // GNG parameters
  double m_paramAgeIncrement;
  double m_paramAgeMax;
  double m_paramMoveWinner;
  double m_paramMoveNeighbor;
  double m_paramErrorDecay;
};


#endif

