/**
  * Copyright (c) 2000-2004 Jim Holmstrm. All rights reserved.
  *
  * The author grants you the right to use, modify, and redistribute this
  * software for noncommercial purposes, on the conditions that you:
  * (1) retain the original headers, including the copyright notice and
  * this text, (2) clearly document the difference between any derived
  * software and the original, and (3) acknowledge your use of this
  * software in pertaining publications and reports.  This software
  * is provided "as is", without any warranty of any kind.
  *
  * Author  : Jim Holmstrm
  * E-mail  : jimh@it.uu.se
  *           boll@booru.net
  * Web     : http://user.it.uu.se/~jimh
  *           http://www.booru.net
  *
  */

//-----------------------------------------------------------------------------
// Author: Jim Holmstrm
// Date:   2002-02-13
//-----------------------------------------------------------------------------

#ifndef __GNGNODE_H
#define __GNGNODE_H

#include <Vector.hpp>
#include "EdgeContainer.hpp"

class GNGEdge;

/**
 * Represents a node in the GNG algorithm
 * contains a reference vector, a collection of edge-references
 * and a local accumulated error. ( and a utility )
 * GNGNode does not assume ownership of the memory reserved by the
 * edges it contains.
 */
class GNGNode
{
public:
  GNGNode();
  virtual ~GNGNode();
  
  /**
   * Sets the Reference Vector of the node.
   * @param referenceVector The reference Vector.
   */
  virtual void SetReferenceVector(Vector const & referenceVector);

  /**
   * Gets a non-const reference to the Reference Vector.
   * @return the reference Vector ref.
   */
  virtual Vector & GetReferenceVector();

  /**
   * Gets a const reference to the Reference Vector.
   * @return the reference Vector ref.
   */
  virtual Vector const & GetReferenceVector() const;
  
  /**
   * Sets the accumulated error of the node.
   * @param d the accumulated error.
   */
  virtual void SetAccumulatedError(double d);

  /**
   * Gets the accumulated error of the node.
   * @return the accumulated error.
   */
  virtual double GetAccumulatedError() const;
  
  /**
   * Sets the utility (used by GNG-U) of the node.
   * @param d the utility.
   */
  virtual void SetUtility(double d);

  /**
   * Gets the utility of the node.
   * @return the utility.
   */
  virtual double GetUtility() const;
  
  /**
   * Registers a reference to a GNGEdge with this node.
   * @param edge The edge to add.
   */
  virtual void AddEdge(GNGEdge & edge);

  /**
   * Unregisters the reference to a GNGEdge with this node.
   * @param edge The edge to remove.
   */
  virtual void RemoveEdge(GNGEdge const & edge);

  /**
   * Gets a pointer to an Edge if this node is connected to <node>
   * @param node The node to which this node might be connected.
   * @return a pointer to the GNGEdge if the nodes are connected and
   * null-pointer if not.
   */
  virtual GNGEdge * GetEdgeTo(GNGNode const & node);

  /**
   * Gets a constant reference to the Edges of this node.
   * @return the reference to this nodes edges.
   */
  virtual EdgeContainer const & GetEdges() const;
  
  /**
   * Gets a reference to the Edges of this node.
   * @return the reference to this nodes edges.
   */
  virtual EdgeContainer & GetEdges();

protected:
  Vector         m_referenceVector;    
  EdgeContainer  m_edges;
  double         m_accumulatedError;
  double         m_utility;
  
};


class GNGNodeFactory
{
public:
  virtual GNGNode * NewNode() { return new GNGNode(); }
};

#endif
