/**
  * Copyright (c) 2000-2004 Jim Holmstrm. All rights reserved.
  *
  * The author grants you the right to use, modify, and redistribute this
  * software for noncommercial purposes, on the conditions that you:
  * (1) retain the original headers, including the copyright notice and
  * this text, (2) clearly document the difference between any derived
  * software and the original, and (3) acknowledge your use of this
  * software in pertaining publications and reports.  This software
  * is provided "as is", without any warranty of any kind.
  *
  * Author  : Jim Holmstrm
  * E-mail  : jimh@it.uu.se
  *           boll@booru.net
  * Web     : http://user.it.uu.se/~jimh
  *           http://www.booru.net
  *
  */

//-----------------------------------------------------------------------------
// Author: Jim Holmstrm
// Date:   2002-02-13
//-----------------------------------------------------------------------------

#include <Exception.hpp>
#include <Vector.hpp>
#include <Random.hpp>

#include "GNGUAlgorithm.hpp"
#include "NodeContainer.hpp"
#include "EdgeContainer.hpp"
#include "GNGEdge.hpp"
#include "GNGNode.hpp"
#include "IGNGContainer.hpp"
#include "IGNGNodeInserter.hpp"

#include <cstdlib>
#include <iostream>

using namespace std;


//-----------
void GNGUAlgorithm::Iterate(Vector const & input)
{
  if(!m_initDone)
    throw Exception("Iterate::Init must be called first.");

  m_input = input;

  // Step 2
  //   locate two nodes closest to the input.
  GNGNode * s = 0;
  GNGNode * t = 0;
  GetClosestNodes(s, t);

  // Step 3
  UpdateError(*s);
  UpdateUtility(*s, *t);  // GNG-U specific

  // Step 4
  //   Increase age of all edges to s's topological neighbors
  IncreaseEdgeAges(*s);

  // Step 5
  MoveWinner(*s);
  MoveNeighbors(*s);

  // Step 6
  GNGEdge * edge = s->GetEdgeTo(*t);
  if( edge != 0 )
    edge->SetAge(0);
  else
    m_gngContainer->CreateEdge(s, t);
    
  // Step 7
  PurgeEdges();
  PurgeNodes();

  // Step 8
  if( m_nodeInserter->IsInsertCriteriaMet() )
  {
    m_nodeInserter->InsertNewNode();
  }

  // Step 9
  DecreaseNodeErrors();
  DecreaseNodeUtiliy(); // GNG-U specific

}


//-----------
void GNGUAlgorithm::PurgeNodes()
{
  NodeContainer & nodes = m_gngContainer->GetNodes();
  
  NodeContainer::iterator it = nodes.begin();
  NodeContainer::iterator utilityIterator = nodes.begin();

  double maxError = (*it)->GetAccumulatedError();;
  double minUtility = (*it)->GetUtility();

  // find max error and min utility
  while(it != nodes.end())
  {
    GNGNode & node = *(*it);
    
	  double error = node.GetAccumulatedError();
	  if( error > maxError )
	    maxError = error;

	  double utility = node.GetUtility();
    if( utility < minUtility )
    {
	    minUtility = utility;
	    utilityIterator = it;
	  }

    it++;
  }

  if(minUtility != 0.0 && nodes.size() > 2)
  if(maxError > minUtility * m_paramUtility)  //if(maxError / minUtility > m_paramUtility)  
  {
    GNGNode * n = (*utilityIterator);
    
    cout << "[delete] " 
         << "max E: " << maxError << ", "
         << "min U: " << minUtility  << ", "
         << "E/U: " << maxError / minUtility
         << endl;
    
    // remove the nodes edges.
    EdgeContainer & nodeEdges = n->GetEdges();
    EdgeContainer::iterator it = nodeEdges.begin();
    
    while(it != nodeEdges.end())
    {
      GNGEdge & e = *(*it);      
      e.GetOtherNode(*n).RemoveEdge(e); // remove edge from node at other end
      nodeEdges.erase(it);              // remove edge from n.
      m_gngContainer->DeleteEdge(&e);   // remove from node container and frees memory.
    }
    
    // remove the node.
    m_gngContainer->DeleteNode(utilityIterator);

  }

}


//-----------
void GNGUAlgorithm::UpdateUtility(GNGNode & s, GNGNode & t)
{
	double u = s.GetUtility();
	u = u + m_input.SquaredDistance(t.GetReferenceVector()) - m_input.SquaredDistance(s.GetReferenceVector());
	s.SetUtility( u );
}


//-----------
void GNGUAlgorithm::DecreaseNodeUtiliy()
{
  NodeContainer & nodes = m_gngContainer->GetNodes();
  for(NodeContainer::iterator it = nodes.begin(); it != nodes.end(); it++)
  {
    GNGNode & s = *(*it);
	  double u = s.GetUtility() * (1.0 - m_paramUtilityDecay);
	  s.SetUtility( u );   
  }
}

