/**
  * Copyright (c) 2000-2004 Jim Holmstrm. All rights reserved.
  *
  * The author grants you the right to use, modify, and redistribute this
  * software for noncommercial purposes, on the conditions that you:
  * (1) retain the original headers, including the copyright notice and
  * this text, (2) clearly document the difference between any derived
  * software and the original, and (3) acknowledge your use of this
  * software in pertaining publications and reports.  This software
  * is provided "as is", without any warranty of any kind.
  *
  * Author  : Jim Holmstrm
  * E-mail  : jimh@it.uu.se
  *           boll@booru.net
  * Web     : http://user.it.uu.se/~jimh
  *           http://www.booru.net
  *
  */

//-----------------------------------------------------------------------------
// Author: Jim Holmstrm
// Date:   2002-02-13
//-----------------------------------------------------------------------------

#ifndef __INPUTGENERATORDEFAULTNONSTATIONARY_H
#define __INPUTGENERATORDEFAULTNONSTATIONARY_H

#include <Random.hpp>
#include "IInputGenerator.hpp"
#include <GNGRepresentation/IGLInputRepresentation.hpp>

#include <cstdlib>
#include <GL/glut.h>

/**
 * Moves a provided inputGenerator in a square-path counter clockwise fashion.
 */
class InputGeneratorDefaultNonstationary : public IInputGenerator
{ 
public:
  InputGeneratorDefaultNonstationary(IInputGenerator * ig, double stepSize, double scale) 
  : m_inputGenerator(*ig)
  {
    m_stepSize = stepSize; // move stepSize each GetInput call
    m_scale = scale;

    m_minX = -1 + m_scale;
    m_minY = -1 + m_scale;
    m_maxX = 1 - m_scale;
    m_maxY = 1 - m_scale;
  
    // The movement of the distribution is represented by a position along the
    //   edge of the square (maxX, maxY) to (minX, minY)
    m_translate.push_back(m_maxX);  // start x-position   
    m_translate.push_back(m_maxY);  // start y-position

    m_dir.push_back(0.0);
    m_dir.push_back(0.0);
    
    m_vector.push_back(0.0);
    m_vector.push_back(0.0);
  }

  virtual ~InputGeneratorDefaultNonstationary() 
	{ 
		delete &m_inputGenerator; 
	}
  
  /**
   * @return An m_dimensional vector.
   */
  virtual Vector GetInput()
  {
    //m_vector[0] = Random::Randomize() * m_scale / 2.0 + m_translate[0];
    //m_vector[1] = Random::Randomize() * m_scale / 2.0 + m_translate[1];

    // Get input from the input generator, scale and translate.
    m_vector = (m_inputGenerator.GetInput() * m_scale) + m_translate;
    
    // left
    if(m_translate[1] >= m_maxY && m_translate[0] > m_minX)
    {
      m_dir[0] = -m_stepSize;
      m_dir[1] = 0.0;
    }
    else // down
    if(m_translate[1] > m_minY && m_translate[0] <= m_minX)
    {
      m_dir[0] = 0.0;
      m_dir[1] = -m_stepSize;
    }
    else // right
    if(m_translate[1] <= m_minY && m_translate[0] < m_maxX)
    {
      m_dir[0] = m_stepSize;
      m_dir[1] = 0.0;
    }
    else // up
    if(m_translate[1] < m_maxY && m_translate[0] >= m_maxX)
    {
      m_dir[0] = 0.0;
      m_dir[1] = m_stepSize;
    }
    
    m_translate = m_translate + m_dir;

    return m_vector;
  }

  /**
   * Returns the output corresponding to the latest generated input.
   * Must be implemented, but is not used by GNG or GNG-U.
   * @return the output corresponding to the latest generated input.
   */
  virtual Vector GetOutput()
  {
    return m_inputGenerator.GetOutput();
  }
 
  /**
   * @return The Latest generated Input Vector.
   */
  virtual Vector const & GetLastInput() const { return m_vector; }  

  /**
   * @return The dimension, which is 2.
   */
  virtual unsigned int GetDimension() const { return m_inputGenerator.GetDimension(); }

  /**
   * @return The current x-position of the moving square.
   */
  Vector GetTranslation() { return m_translate; }

  /**
   * @return The size of the edges of the moving square.
   */
  double GetScale() { return m_scale; }

  /**
   * @return ClassesGenerator, since this is a generator for classification
   */
  virtual GeneratorType GetGeneratorType() const { return ClassesGenerator; }

  
protected:
  double m_scale;
  double m_stepSize;
  Vector m_dir;
  Vector m_translate;
  Vector m_vector;  
  IInputGenerator & m_inputGenerator;
  double m_minX;
  double m_minY;
  double m_maxX;
  double m_maxY;
};


/**
 * An openGL representation. 
 */ 
class InputRepresentationDefaultNonstationary : public IGLInputRepresentation
{
public:
  InputRepresentationDefaultNonstationary(InputGeneratorDefaultNonstationary * i, IGLInputRepresentation * ir)
  : m_inputGenerator(*i), m_inputRepresentation(*ir) { }
  
  virtual ~InputRepresentationDefaultNonstationary() 
	{ 
		delete &m_inputRepresentation; 
	}
  
  virtual void GLOutputDistribution() const
  {
    Vector translate(2);
    translate = m_inputGenerator.GetTranslation();
    double scale = m_inputGenerator.GetScale();

    glPushMatrix();
     glTranslatef(translate[0], translate[1], 0);
     glScalef(scale, scale, 0);
     m_inputRepresentation.GLOutputDistribution();
    glPopMatrix();
  }
  
  virtual void GLOutputSignal() const
  {
    // draw the last input point.
    Vector const & v = m_inputGenerator.GetLastInput();
    glBegin(GL_POINTS);
    {			
      glVertex2d(v[0], v[1]);
    }    
    glEnd();
  }
  
private:
  InputGeneratorDefaultNonstationary & m_inputGenerator;
  IGLInputRepresentation &             m_inputRepresentation;
};

#endif

