/**
  * Copyright (c) 2000-2004 Jim Holmstrm. All rights reserved.
  *
  * The author grants you the right to use, modify, and redistribute this
  * software for noncommercial purposes, on the conditions that you:
  * (1) retain the original headers, including the copyright notice and
  * this text, (2) clearly document the difference between any derived
  * software and the original, and (3) acknowledge your use of this
  * software in pertaining publications and reports.  This software
  * is provided "as is", without any warranty of any kind.
  *
  * Author  : Jim Holmstrm
  * E-mail  : jimh@it.uu.se
  *           boll@booru.net
  * Web     : http://user.it.uu.se/~jimh
  *           http://www.booru.net
  *
  */

#ifndef __INPUTGENERATORDISCRETE_H
#define __INPUTGENERATORDISCRETE_H

#include "IInputGenerator.hpp"

#include <fstream>
#include <iostream>
#include <Vector.hpp>

/**
* Reads an image in raw 8bit format. Each pixel is 8 bits.
* All read pixels will be classified and normalized.
*/
class RawReader
{
public:

   //--------
   class Point 
   {
   public:
     Vector m_pos;
	 Vector m_class;
   };
   
   typedef std::vector<Point> Points;
   
   //--------
   RawReader(std::string const & filename, int width, int height)
   {
     m_fileName = filename;
	 m_width = width;
	 m_height = height;
   }
      
   //--------
   Points * Convert()
   {
      std::ifstream * file = new std::ifstream(m_fileName.c_str());
 
      int numClasses = 0;
      while(! file->eof() )
      {
        int data = file->get();
        if(data > numClasses) 
           numClasses = data;
      }

	  file->close();
	  delete file;
	  file = 0;
	  
      file = new std::ifstream(m_fileName.c_str());

      Points * points = new Points;
      unsigned int index = 0;
      while(! file->eof() )
      {
        int data = file->get();

		if(data > 0)
		{
		  double x = ((double)(index % m_width) / (double)m_width) * 2.0 - 1.0;
		  double y = ((double)((int)(index / m_height)) / (double)m_height) * 2.0 - 1.0;
		
		  Point point;
		  point.m_pos.push_back( x );
		  point.m_pos.push_back( y );

		  // create the output vector
		  for(int i=0; i<numClasses; i++)
		    point.m_class.push_back(0.0);

		  point.m_class[data-1]= 1.0;

		  points->push_back(point);        
		} 
        index++;
      }
      
	  std::cerr << "Number of Discrete points:" << points->size() << std::endl;
      return points;
   }
   
protected:
   int              m_width;
   int              m_height;   
   std::string      m_fileName;
};



class InputGeneratorDiscrete : public IInputGenerator
{
public: 
  InputGeneratorDiscrete(RawReader::Points * points)
  {
	m_points = points;
    m_dimension = 2;
    m_min = 0.0;
    m_max = 1.0;

    for(unsigned int i=0; i<(*points)[0].m_pos.size(); i++)
      m_input.push_back(0.0);

    for(unsigned int i=0; i<(*points)[0].m_class.size(); i++)
      m_output.push_back(0.0);
  }

  /**
   * @return a vector representing the xor boolean funtion.
   */
  virtual Vector GetInput()
  {
    unsigned int index = (unsigned int)(Random::RandomizePositive() * m_points->size());
    if(index == m_points->size()) 
      index--;

    m_input = (*m_points)[index].m_pos;
    m_output = (*m_points)[index].m_class;

    return m_input;
  }
    
  /**
   * @return the 1-dim class for the current input, 0 or 1
   */
  virtual Vector GetOutput()
  {
    return m_output;
  }

  /**
   * @return The Latest generated Input Vector.
   */
  virtual Vector const & GetLastInput() const { return m_input; }  

  /**
   * @return The dimension, which is deciced at construction time.
   */
  virtual unsigned int GetDimension() const { return m_dimension; }
  
  virtual RawReader::Points const & GetPoints() { return *m_points; }

  /**
   * @return ClassesGenerator, since this is a generator for classification
   */
  virtual GeneratorType GetGeneratorType() const { return ClassesGenerator; }
  
protected:
  unsigned int m_dimension;
  double m_min;
  double m_max;
  Vector m_input;
  RawReader::Points * m_points;
};


/**
 * An openGL representation. A class that represents the InputGeneratorDiscrete domain.
 */ 
class InputRepresentationDiscrete : public IGLInputRepresentation
{
public:
  InputRepresentationDiscrete(InputGeneratorDiscrete * i) : m_inputGenerator(*i) {}
  ~InputRepresentationDiscrete() {}

  virtual void GLOutputDistribution() const
  {
    RawReader::Points const & p = m_inputGenerator.GetPoints();
    glBegin(GL_POINTS);
    {			
      for(unsigned int i=0; i<p.size(); i++)
      {
        unsigned int classIndex=0;
        for(; classIndex<p[0].m_class.size(); classIndex++)
          if(p[i].m_class[classIndex] == 1.0)
            break;
          
        glColor4f(0.5 + ((double)classIndex)*0.2, 0.5 + ((double)classIndex)*0.2, 0.4, 1.0);
          
        double x = p[i].m_pos[0];
        double y = p[i].m_pos[1];
        
        glVertex2d(x, y);
      }
    }    
    glEnd();
  }
  
  virtual void GLOutputSignal() const 
  {    
    Vector const & v = m_inputGenerator.GetLastInput();
    glBegin(GL_POINTS);
    {			
      glVertex2d(v[0], v[1]);
    }    
    glEnd();
  }
  
private:
  InputGeneratorDiscrete & m_inputGenerator;
};

#endif
