/**
  * Copyright (c) 2000-2004 Jim Holmstrm. All rights reserved.
  *
  * The author grants you the right to use, modify, and redistribute this
  * software for noncommercial purposes, on the conditions that you:
  * (1) retain the original headers, including the copyright notice and
  * this text, (2) clearly document the difference between any derived
  * software and the original, and (3) acknowledge your use of this
  * software in pertaining publications and reports.  This software
  * is provided "as is", without any warranty of any kind.
  *
  * Author  : Jim Holmstrm
  * E-mail  : jimh@it.uu.se
  *           boll@booru.net
  * Web     : http://user.it.uu.se/~jimh
  *           http://www.booru.net
  *
  */

//-----------------------------------------------------------------------------
// Author: Jim Holmstrm
// Date:   2002-02-13
//-----------------------------------------------------------------------------

#ifndef __INPUTGENERATORFOUR_H
#define __INPUTGENERATORFOUR_H

#include <Random.hpp>
#include "IInputGenerator.hpp"
#include <GNGRepresentation/IGLInputRepresentation.hpp>

#include <cstdlib>
#include <cmath>
#include <GL/glut.h>

/**
 * Presents 4 versions in square formation of the provided inputGenerator.
 */
class InputGeneratorFour : public IInputGenerator
{
public:
  InputGeneratorFour(IInputGenerator * ig, double scale) : m_inputGenerator(*ig)
  {
    m_dimension = m_inputGenerator.GetDimension();
    if(m_dimension != 2)
      throw Exception("FourInput::Constructor:: Illegal dimension. Must be 2");

    m_scale = scale;
    
    m_vector.push_back(0.0);
    m_vector.push_back(0.0);

    Vector v = ig->GetOutput();
    unsigned int theSize = 4*v.size();
    for(unsigned int i=0; i<theSize; i++)
        m_output.push_back(0.0);    
  }

  virtual ~InputGeneratorFour() 
  { 
    delete &m_inputGenerator; 
  }
  
  /**
  * @return An m_dimensional vector.
  */
  virtual Vector GetInput()
  {
    Vector v(m_dimension);

    // Get an input signal from the IG. and scale it.
    v = m_inputGenerator.GetInput() * m_scale;
        
    // Decide which quadrant it should be in and create the translation
    double quadrant = Random::RandomizePositive();
    Vector translation(m_dimension);
    
    Vector output = m_inputGenerator.GetOutput();
    unsigned int theClass = 0;
	unsigned int outputSize = output.size();
	
    for(unsigned int i=0; i<m_output.size(); i++)
      m_output[i] = 0.0;            
      
    for(unsigned int i=0; i<outputSize; i++)
    {
      if(output[i] == 1.0)
      {
        theClass = i;
        break;
      }
    }
 

    //quadrant 0
    if(quadrant < 0.25)
    {
      translation[0] = 1 - m_scale;
      translation[1] = 1 - m_scale;
      m_output[theClass] = 1.0;            
    }    
    else
    //quadrant 1
    if(quadrant < 0.50)
    {
      translation[0] = -1 + m_scale;
      translation[1] = 1 - m_scale;
      m_output[1 * outputSize + theClass] = 1.0;            
    }    
    else
    //quadrant 2
    if(quadrant < 0.75)
    {
      translation[0] = -1 + m_scale;
      translation[1] = -1 + m_scale;
      m_output[2 * outputSize + theClass] = 1.0;            
    }    
    else
    //quadrant 3
    if(quadrant <= 1.0)
    {
      translation[0] = 1 - m_scale;
      translation[1] = -1 + m_scale;
      m_output[3 * outputSize + theClass] = 1.0;            
    }    
       
    m_vector = v + translation;     
    return m_vector;
  }
  
	virtual Vector GetOutput() 
	{ 
		return m_output; 
	}

  /**
   * @return The Latest generated Input Vector.
   */
  virtual Vector const & GetLastInput() const { return m_vector; }

  /**
   * @return The dimension. 
   */
  virtual unsigned int GetDimension() const { return m_inputGenerator.GetDimension(); }

  /**
   * @return The displacement-constant of the squares.
   */
  double GetScale() { return m_scale; }

  /**
   * @return ClassesGenerator, since this is a generator for classification
   */
  virtual GeneratorType GetGeneratorType() const { return ClassesGenerator; }

protected:
  unsigned int         m_dimension;
  double               m_scale;
  IInputGenerator &    m_inputGenerator;
  Vector               m_vector;
};


/**
 * An openGL representation. A class that represents the domain of FourInputRepresentation.
 */ 
class InputRepresentationFour : public IGLInputRepresentation
{
public:
  InputRepresentationFour(InputGeneratorFour * i, IGLInputRepresentation * r) 
  : m_inputGenerator(*i), m_inputRepresentation(*r) {}

	virtual ~InputRepresentationFour()
	{
		delete &m_inputRepresentation;
	}
  
  virtual void GLOutputDistribution() const
  {
    // draw the areas of the distribution.
    double d = m_inputGenerator.GetScale();

    // quadrant 0
    glPushMatrix();
     glTranslatef(-1+d,1-d,0);
     glScalef(d,d,0);
     m_inputRepresentation.GLOutputDistribution();
    glPopMatrix();
    // quadrant 1
    glPushMatrix();
     glTranslatef(-1+d,-1+d,0);
     glScalef(d,d,0);
     m_inputRepresentation.GLOutputDistribution();
    glPopMatrix();
    // quadrant 2
    glPushMatrix();
     glTranslatef(1-d,-1+d,0);
     glScalef(d,d,0);
     m_inputRepresentation.GLOutputDistribution();
    glPopMatrix();
    // quadrant 3
    glPushMatrix();
     glTranslatef(1-d,1-d,0);
     glScalef(d,d,0);
     m_inputRepresentation.GLOutputDistribution();
    glPopMatrix();
  }
  
  virtual void GLOutputSignal() const
  {
    // draw the last input point.
    Vector const & v = m_inputGenerator.GetLastInput();
    glBegin(GL_POINTS);
    {			
      glVertex2d(v[0], v[1]);
    }    
    glEnd();
  }
  
private:
  InputGeneratorFour &       m_inputGenerator;
  IGLInputRepresentation &   m_inputRepresentation;
};

#endif

