/**
  * Copyright (c) 2000-2004 Jim Holmstrm. All rights reserved.
  *
  * The author grants you the right to use, modify, and redistribute this
  * software for noncommercial purposes, on the conditions that you:
  * (1) retain the original headers, including the copyright notice and
  * this text, (2) clearly document the difference between any derived
  * software and the original, and (3) acknowledge your use of this
  * software in pertaining publications and reports.  This software
  * is provided "as is", without any warranty of any kind.
  *
  * Author  : Jim Holmstrm
  * E-mail  : jimh@it.uu.se
  *           boll@booru.net
  * Web     : http://user.it.uu.se/~jimh
  *           http://www.booru.net
  *
  */

//-----------------------------------------------------------------------------
// Author: Jim Holmstrm
// Date:   2002-02-13
//-----------------------------------------------------------------------------

#ifndef __INPUTGENERATORJUMP_H
#define __INPUTGENERATORJUMP_H

#include <Random.hpp>
#include "IInputGenerator.hpp"
#include <GNGRepresentation/IGLInputRepresentation.hpp>

#include <cstdlib>
#include <GL/glut.h>

/**
 * Moves a provided inputGenerator from upper left corner to lower right
 */
class InputGeneratorJump : public IInputGenerator
{ 
public:
  InputGeneratorJump(IInputGenerator * ig, unsigned int jumpDelay, double scale) : m_inputGenerator(*ig)
  {
    m_jumpDelay = jumpDelay;
    m_scale = scale;
    m_steps = 0;

    m_minX = -1 + m_scale;
    m_minY = -1 + m_scale;
    m_maxX = 1 - m_scale;
    m_maxY = 1 - m_scale;
  
    m_translate.push_back(m_minX);  // start x-position   
    m_translate.push_back(m_maxY);  // start y-position

    m_dir.push_back(0.0);
    m_dir.push_back(0.0);
    
    m_vector.push_back(0.0);
    m_vector.push_back(0.0);
  }
  
  virtual ~InputGeneratorJump() 
  { 
    delete &m_inputGenerator; 
  }

  /**
   * @return An m_dimensional vector.
   */
  virtual Vector GetInput()
  {
    m_steps++;
    // Get input from the input generator, scale and translate.
    m_vector = (m_inputGenerator.GetInput() * m_scale) + m_translate;
     
    // jump from upper left to lower right and vv.
    if(m_steps % m_jumpDelay == 0)
    {
      m_translate[0] = m_translate[0] * -1.0;
      m_translate[1] = m_translate[1] * -1.0;
    }

    return m_vector;
  }
  
  /**
   * Returns the output corresponding to the latest generated input.
   * Must be implemented, but is not used by GNG or GNG-U.
   * @return the output corresponding to the latest generated input.
   */
  virtual Vector GetOutput()
  {
    return m_inputGenerator.GetOutput();
  }

  
  /**
   * @return The Latest generated Input Vector.
   */
  virtual Vector const & GetLastInput() const { return m_vector; }  

  /**
   * @return The dimension, which is 2.
   */
  virtual unsigned int GetDimension() const { return m_inputGenerator.GetDimension(); }

  /**
   * @return The current x-position of the moving square.
   */
  Vector GetTranslation() { return m_translate; }

  /**
   * @return The size of the edges of the moving square.
   */
  double GetScale() { return m_scale; }

  /**
   * @return ClassesGenerator, since this is a generator for classification
   */
  virtual GeneratorType GetGeneratorType() const { return ClassesGenerator; }
  
protected:
  double m_scale;
  unsigned int m_steps;
  unsigned int m_jumpDelay;
  Vector m_dir;
  Vector m_translate;
  Vector m_vector;  
  IInputGenerator & m_inputGenerator;
  double m_minY;
  double m_maxY;
  double m_minX;
  double m_maxX;
};


/**
 * An openGL representation. 
 */ 
class InputRepresentationJump : public IGLInputRepresentation
{
public:
  InputRepresentationJump(InputGeneratorJump * i, IGLInputRepresentation * ir)
  : m_inputGenerator(*i), m_inputRepresentation(*ir) { }

  virtual ~InputRepresentationJump() 
	{ 
		delete &m_inputRepresentation; 
	}
  
  virtual void GLOutputDistribution() const
  {
    Vector translate(2);
    translate = m_inputGenerator.GetTranslation();
    double scale = m_inputGenerator.GetScale();

    glPushMatrix();
     glTranslatef(translate[0], translate[1], 0);
     glScalef(scale, scale, 0);
     m_inputRepresentation.GLOutputDistribution();
    glPopMatrix();
  }
  
  virtual void GLOutputSignal() const
  {
    // draw the last input point.
    Vector const & v = m_inputGenerator.GetLastInput();
    glBegin(GL_POINTS);
    {			
      glVertex2d(v[0], v[1]);
    }    
    glEnd();
  }
  
private:
  InputGeneratorJump & m_inputGenerator;
  IGLInputRepresentation & m_inputRepresentation;
};

#endif

