/**
  * Copyright (c) 2000-2004 Jim Holmstrm. All rights reserved.
  *
  * The author grants you the right to use, modify, and redistribute this
  * software for noncommercial purposes, on the conditions that you:
  * (1) retain the original headers, including the copyright notice and
  * this text, (2) clearly document the difference between any derived
  * software and the original, and (3) acknowledge your use of this
  * software in pertaining publications and reports.  This software
  * is provided "as is", without any warranty of any kind.
  *
  * Author  : Jim Holmstrm
  * E-mail  : jimh@it.uu.se
  *           boll@booru.net
  * Web     : http://user.it.uu.se/~jimh
  *           http://www.booru.net
  *
  */

#ifndef __INPUTGENERATORSINUS_H
#define __INPUTGENERATORSINUS_H

#include "IInputGenerator.hpp"
#include <cmath>
#include <iostream>

/**
* A generator that generates inputs and corresponding 
* outputs to the Sinus funciton.
*/
class InputGeneratorSinus : public IInputGenerator
{
public: 
  static const double PI;
  
public: 
  InputGeneratorSinus(unsigned int nofPoints, double start, double end)
  {
    m_deltaX = (end - start) / ((double)nofPoints-1.0);
    m_start = start;
    m_end = end;
    
    for(double i=start; i<=end; i+=m_deltaX)
    {
      double sinVal = sin(i);
      m_sin.push_back( sinVal );
      m_x.push_back(i);
      //std::cout << i << " " << sinVal << endl;
    }

    m_dimension = 1;
    m_input.push_back(0.0);
    m_output.push_back(0.0);
  }

  /**
   * @return a 1-dim vector input to the sin function
   */
  virtual Vector GetInput()
  {    
    unsigned int index = (unsigned int)(Random::RandomizePositive() * (double)m_sin.size());
    if(index == m_sin.size()) index--;
    
    m_input[0] = m_x[index];
    m_output[0] = m_sin[index];

    return m_input;
  }
    
  /**
   * @return the 1-dim output for the current input, 0 or 1
   */
  virtual Vector GetOutput()
  {
    return m_output;
  }

  /**
   * @return The Latest generated Input Vector.
   */
  virtual Vector const & GetLastInput() const { return m_input; }  

  /**
   * @return The dimension, which is deciced at construction time.
   */
  virtual unsigned int GetDimension() const { return m_dimension; }

  /**
   * @return ClassesGenerator, since this is a generator for classification
   */
  virtual GeneratorType GetGeneratorType() const { return FunctionGenerator; }

  /**
   * @return the x-values used.
   */
  virtual Vector const & GetXValues() const { return m_x; }


  virtual double GetStart() const { return m_start; }
  virtual double GetEnd() const { return m_end; }
    

protected:
  unsigned int m_dimension;
  Vector m_sin;
  Vector m_x;
  double m_start;
  double m_end;
  double m_deltaX;
  Vector m_input;  
};

const double InputGeneratorSinus::PI = 3.14159265358979;

/**
 * An openGL representation. A class that represents the InputGeneratorSinus domain.
 */ 
class InputRepresentationSinus : public IGLInputRepresentation
{
public:
  InputRepresentationSinus(InputGeneratorSinus * i) : m_inputGenerator(*i) {}
	~InputRepresentationSinus() {}

  virtual void GLOutputDistribution() const
  {
    Vector const & v = m_inputGenerator.GetXValues();
        
    glBegin(GL_POINTS);
    {
      for(unsigned int i=0; i<v.size(); i++)
        glVertex2d(v[i], 0);
    }    
    glEnd();
  }
  
  virtual void GLOutputSignal() const 
  {    
    Vector const & v = m_inputGenerator.GetLastInput();
    glBegin(GL_POINTS);
    {			
      glVertex2d(v[0],0.0);
    }    
    glEnd();
  }

  /**
  * Called by the GNGGLUT representation to adjust the screen size.
  * @return the max x size of the GLUT representation.
  */
  virtual double GetDisplayXMax() { return m_inputGenerator.GetEnd(); }  
  virtual double GetDisplayYMax() { return 0.5; }  
  virtual double GetDisplayXMin() { return m_inputGenerator.GetStart(); }  
  virtual double GetDisplayYMin() { return -0.5; }  

  
private:
  InputGeneratorSinus & m_inputGenerator;
};


#endif
