/**
  * Copyright (c) 2000-2004 Jim Holmstrm. All rights reserved.
  *
  * The author grants you the right to use, modify, and redistribute this
  * software for noncommercial purposes, on the conditions that you:
  * (1) retain the original headers, including the copyright notice and
  * this text, (2) clearly document the difference between any derived
  * software and the original, and (3) acknowledge your use of this
  * software in pertaining publications and reports.  This software
  * is provided "as is", without any warranty of any kind.
  *
  * Author  : Jim Holmstrm
  * E-mail  : jimh@it.uu.se
  *           boll@booru.net
  * Web     : http://user.it.uu.se/~jimh
  *           http://www.booru.net
  *
  */

#ifndef __RBFCLASSIFICATIONCONTROL_H
#define __RBFCLASSIFICATIONCONTROL_H

#include <Control.hpp>
#include <GNGCore/SGNGAlgorithm.hpp>
#include "RBFNetwork.hpp"
#include <Generators/IInputGenerator.hpp>

#include <iostream>
#include <fstream>

// the difference between the larget answer and 
//  second largest answer in the output vector.
#define RBF_DECISION_DELTA (0.25)

/**
* GNGGLUTRepresentation uses the control-object's
* Iterate method every time the CallBackIdleFunc is called.
* Placing the code separately in this object is a way
* of allowing GL independent code and state information.
*/
class RBFClassificationControl : public Control
{
public:
	/**
	* Creates a RBF Network control Object that "runs" the network and sgng alg.
	* @param net The RBF network
	* @param g a Generator object that generates input and corresponding output signals.
	* @param alg The SGNG version of GNG
	* @param max The maximum number of misclassifications allowed, when below this, we stop.
	* @param backLog The log size, used to calculate the current number of misclassifications.
	*/
	RBFClassificationControl(RBFNetwork & net, IInputGenerator * g, SGNGAlgorithm * alg, 
		unsigned int max, unsigned int backLog, double decisionLimit, double decisionDelta, std::ofstream * log) 
		: m_net(net), m_generator(*g), m_alg(alg)
	{
		m_decisionLimit = decisionLimit;
		m_decisionDelta = decisionDelta;
		m_maxMisclassificationsAllowed = max;
		m_numMisclassifications = m_maxMisclassificationsAllowed+1;
		m_backLog = backLog;
		m_iteration = 0;

		m_log = log;
	}

	virtual ~RBFClassificationControl() 
	{ 
		delete m_alg; 
		delete &m_generator; 
		m_log->close();
		delete m_log;
	}

	/**
	* The main loop.
	*/
	virtual bool Iterate()
	{
		static unsigned int backLog = m_backLog;
		static unsigned int numMisclassifications = 0;

		if(IsStopCriteriaMet())
			return false;

		if(IsReadyToIterate())
		{
			m_iteration++;
			backLog--;

			Vector input = m_generator.GetInput();
			Vector output = m_generator.GetOutput();

			std::cout << m_iteration;
			std::cout << ", I:[" << input << "], D:[" << output << "]";
			Vector netOutput = m_net.Recall( input );

			
			// decide if the output is correct. 
			//  For a correct answer to be accepted, the differece
			//  between the answer and second best answer must be
			//  at least m_decisionLimit.
			//  Find the best and second best.
			unsigned int bestIndex = 0;
			unsigned int secondBestIndex = 1;
			for(unsigned int i=1; i < netOutput.size(); i++)
			{
			  if(netOutput[i] > netOutput[bestIndex])
			  {
			    secondBestIndex = bestIndex;
			    bestIndex = i;
			  }
			  else
			    if(netOutput[i] > netOutput[secondBestIndex])
			      secondBestIndex = i;
			}

			std::cout << ", O:[" << netOutput << "]";

			// The Decision Limit check.
			//   Reject the answer if diff between best and second best is
			//   less than m_decisionDelta or of best answer is too low.                                             
			if( output[bestIndex] == 1.0 &&
					netOutput[bestIndex] >= m_decisionLimit && 
					fabs(netOutput[bestIndex] - netOutput[secondBestIndex]) >= m_decisionDelta )
			{
				std::cout << ", 1 "; //<< winner; //netOutput;
			}
			else
			{
				numMisclassifications++;
				std::cout << ", 0 " ;
			}
			std::cout << std::endl;

			// train the network
			m_net.Train( output );

			m_alg->SetSquaredError( netOutput.SquaredDistance(output) );
			m_alg->Iterate( input );

			// calculate the set the number of misclassifications in the last backlog steps.
			if(backLog == 0)
			{
				m_numMisclassifications = numMisclassifications;
				backLog = m_backLog;
				numMisclassifications = 0;
				(*m_log) << m_iteration << " " << m_numMisclassifications << std::endl;
			}

			return true;
		}
		return false;
	}

protected:
	virtual bool IsStopCriteriaMet() 
	{
		return m_numMisclassifications <= m_maxMisclassificationsAllowed;
	}

protected:
	RBFNetwork &       m_net;
	IInputGenerator &  m_generator;
	SGNGAlgorithm *    m_alg;

	unsigned int       m_maxMisclassificationsAllowed; // in backlog steps.
	double             m_decisionLimit;
	double             m_decisionDelta;
	unsigned int       m_backLog;
	unsigned int       m_numMisclassifications;
	std::ofstream *    m_log;
};


#endif
