/**
  * Copyright (c) 2000-2004 Jim Holmstrm. All rights reserved.
  *
  * The author grants you the right to use, modify, and redistribute this
  * software for noncommercial purposes, on the conditions that you:
  * (1) retain the original headers, including the copyright notice and
  * this text, (2) clearly document the difference between any derived
  * software and the original, and (3) acknowledge your use of this
  * software in pertaining publications and reports.  This software
  * is provided "as is", without any warranty of any kind.
  *
  * Author  : Jim Holmstrm
  * E-mail  : jimh@it.uu.se
  *           boll@booru.net
  * Web     : http://user.it.uu.se/~jimh
  *           http://www.booru.net
  *
  */

#ifndef __RBFFUNCTIONAPPROXIMATIONCONTROL_H
#define __RBFFUNCTIONAPPROXIMATIONCONTROL_H

#include <Control.hpp>
#include <GNGCore/SGNGAlgorithm.hpp>
#include "RBFNetwork.hpp"
#include <Generators/IInputGenerator.hpp>

#include <iostream>
#include <fstream>

/**
* GNGGLUTRepresentation uses the control-object's
* Iterate method every time the CallBackIdleFunc is called.
* Placing the code separately in this object is a way
* of allowing GL independent code and state information.
*/
class RBFFunctionApproximationControl : public Control
{
public:
	/**
	* Creates a RBF Network control Object that "runs" the network and sgng alg.
	* @param net The RBF network
	* @param g a Generator object that generates input and corresponding output signals.
	* @param alg The SGNG version of GNG
	* @param max The maximum squared error allowed, when below this, we stop. a number between 0 and 1
	* @param backLog The log size, used to calculate the current squared error.
	*/
	RBFFunctionApproximationControl(RBFNetwork & net, IInputGenerator * g, SGNGAlgorithm * alg, double max, unsigned int backLog, std::ofstream * log) 
    : m_net(net), m_generator(*g), m_alg(alg)
	{
		m_maxErrorAllowed = max;
		m_currentError = m_maxErrorAllowed+1;
 		m_backLog = backLog;
 		m_iteration = 0;
		
		m_log = log; 
	}

	virtual ~RBFFunctionApproximationControl() 
	{ 
		delete m_alg; 
		delete &m_generator; 
		m_log->close();
		delete m_log;
	}

	/**
	* The main loop.
	*/
	virtual bool Iterate()
	{
		static unsigned int backLog = m_backLog;
	  static double loggedError = 0.0;
	    
	  if(IsStopCriteriaMet())
	  	return false;
	      
		if(IsReadyToIterate())
		{
			m_iteration++;
			backLog--;
			Vector input = m_generator.GetInput();
			Vector output = m_generator.GetOutput();

			std::cout << m_iteration;
			std::cout << ", I:" << input << ", D:" << output;
			Vector netOutput = m_net.Recall( input );
			std::cout << ", O:" <<  netOutput;
   
			m_net.Train( output );

			double currentError = netOutput.SquaredDistance(output);
			std::cout << ", sqE:" <<  currentError << std::endl;
			loggedError += currentError;
			m_alg->SetSquaredError( currentError );
			m_alg->Iterate( input );

			// calculate the set the mean squared error in the last backlog steps.
			if(backLog == 0)
			{
			   m_currentError = loggedError / (double)m_backLog;
			   backLog = m_backLog;
			   loggedError = 0.0;
			   (*m_log) << m_iteration << " " << m_currentError << std::endl;
			}

			return true;
		}
		return false;
	}

	/**
    * Saves a predefined resolution (and range) discrete input-output table for the current
    * function to disc.
    */
    virtual void Snapshot() 
    {
      // this is just a short hack to allow me to get the sinus approximation to file.
      // the idea is that the range, increment and inputs be handled by the generators
      // but since it is not high prority, I leave that to you to implement.
      /*
      ofstream out("log_SGNG_function.txt");
      double increment = 8.8/10000.0;
      Vector x(1);
      for(x[0]=-1.0; x[0]<7.8; x[0]+=increment)
      {
	    Vector netOutput = m_net.Recall( x );
	    out << x[0] << " " << netOutput[0] << endl;
      }	        
      out.close();
      */
    }

protected:
    virtual bool IsStopCriteriaMet() 
    {
        return m_currentError <= m_maxErrorAllowed;
    }

protected:
	RBFNetwork &       m_net;
	IInputGenerator &  m_generator;
	SGNGAlgorithm *    m_alg;
	
	unsigned int       m_backLog;
	double             m_maxErrorAllowed; // in backlog steps.
    double             m_currentError;
	std::ofstream *    m_log;
};


#endif
