/**
  * Copyright (c) 2000-2004 Jim Holmstrm. All rights reserved.
  *
  * The author grants you the right to use, modify, and redistribute this
  * software for noncommercial purposes, on the conditions that you:
  * (1) retain the original headers, including the copyright notice and
  * this text, (2) clearly document the difference between any derived
  * software and the original, and (3) acknowledge your use of this
  * software in pertaining publications and reports.  This software
  * is provided "as is", without any warranty of any kind.
  *
  * Author  : Jim Holmstrm
  * E-mail  : jimh@it.uu.se
  *           boll@booru.net
  * Web     : http://user.it.uu.se/~jimh
  *           http://www.booru.net
  *
  */

#ifndef __RBFNETWORK_H
#define __RBFNETWORK_H

#include <Vector.hpp>
#include <iostream>
#include "RBFHiddenLayer.hpp"
#include "RBFOutputLayer.hpp"

/**
* A Radial Basis Function Network.
*/
class RBFNetwork
{
public:
	/**
	* Creates a RBF network with nofInputs inputs, one hidden layer and an output layer.
	* @param nofInputs the number of inputs to the network
	* @param hidden the hidden layer
	* @param output the output layer
	*/
	RBFNetwork(/*unsigned int nofInputs, */RBFHiddenLayer * h, RBFWeightLayer * w, RBFOutputLayer * o)
	{
		m_hiddenLayer = h;
		m_weightLayer = w;
		m_outputLayer = o;

		//m_inputs = Vector(nofInputs);
		m_latestOutput = Vector(m_outputLayer->GetNofOutputs());
		m_eta = 0.2; //default
	}

	/**
	* Sets the Eta parameter of the network
	* @param eta the Eta parameter.
	*/
	void SetParameterEta(double eta) { m_eta = eta; }

	/**
	* Presents an input vector and returns the output vector of the RBF network.
	* @param input the input vector.
	* @return the output vector.
	*/
	virtual Vector Recall(Vector const & input)
	{
		m_hiddenLayer->ApplyInput(input);
		Vector const & v = m_outputLayer->GetOutput();
		m_latestOutput = v;
		return v;
	}

	/**
	* Trains the RBF network on a given output.
	* @param output the output vector.
	*/
	virtual void Train(Vector const & output)
	{
		Vector etaDiff(output.size());

		Vector::iterator e = etaDiff.begin();
		Vector::iterator y = m_latestOutput.begin();
		Vector::const_iterator d = output.begin();

		for(; d < output.end(); d++, y++, e++)
			(*e) = m_eta * ( (*d)-(*y) );   // stepsize * (desired_output - actual_output)

		// 1. train weights
		m_weightLayer->UpdateWeights(etaDiff);

		// 2. train bias		
		m_outputLayer->UpdateBias(etaDiff);
	}

private:
	//Vector  m_inputs;
	Vector  m_latestOutput;
	double  m_eta;

	RBFHiddenLayer * m_hiddenLayer;
	RBFWeightLayer * m_weightLayer;
	RBFOutputLayer * m_outputLayer;
};


#endif
