/**
  * Copyright (c) 2000-2004 Jim Holmstrm. All rights reserved.
  *
  * The author grants you the right to use, modify, and redistribute this
  * software for noncommercial purposes, on the conditions that you:
  * (1) retain the original headers, including the copyright notice and
  * this text, (2) clearly document the difference between any derived
  * software and the original, and (3) acknowledge your use of this
  * software in pertaining publications and reports.  This software
  * is provided "as is", without any warranty of any kind.
  *
  * Author  : Jim Holmstrm
  * E-mail  : jimh@it.uu.se
  *           boll@booru.net
  * Web     : http://user.it.uu.se/~jimh
  *           http://www.booru.net
  *
  */

#include <RBF/RBFWeights.hpp>
#include <RBF/RBFWeightLayer.hpp>
#include <RBF/RBFOutputLayer.hpp>
#include <RBF/RBFHiddenLayer.hpp>
#include <RBF/RBFNode.hpp>
#include <RBF/RBFNetwork.hpp>
#include <RBF/RBFClassificationControl.hpp>
#include <RBF/RBFFunctionApproximationControl.hpp>
#include <RBF/IActivation.hpp>
#include <RBF/LinearActivation.hpp>

#include <GNGCore/GNGEdge.hpp>
#include <GNGCore/GNGNode.hpp>
#include <GNGCore/GNGAlgorithm.hpp>
#include <GNGCore/GNGUAlgorithm.hpp>
#include <GNGCore/SGNGAlgorithm.hpp>
#include <GNGCore/GNGControl.hpp>

#include <GNGComponents/DefaultGNGContainer.hpp>
#include <GNGComponents/DefaultNodeMovement.hpp>
#include <GNGComponents/DefaultNodeInserter.hpp>
#include <GNGComponents/SGNGNodeInserter.hpp>

#include <Generators/IInputGenerator.hpp>
#include <Generators/InputGeneratorDefault.hpp>
#include <Generators/InputGeneratorFour.hpp>
#include <Generators/InputGeneratorXOR.hpp>
#include <Generators/InputGeneratorTwoClasses.hpp>
#include <Generators/InputGeneratorDefaultNonstationary.hpp>
#include <Generators/InputGeneratorUpDown.hpp>
#include <Generators/InputGeneratorJump.hpp>
#include <Generators/InputGeneratorDiscrete.hpp>
#include <Generators/InputGeneratorSinus.hpp>


#include <GNGRepresentation/GNGRepresentation.hpp>
#include <GNGRepresentation/GNGGLUTRepresentation.hpp>

#include <Exception.hpp>
#include <Settings.hpp>
#include <Control.hpp>

#include <fstream>
#include <iostream>
using namespace std;

typedef pair<IInputGenerator *, IGLInputRepresentation *> GeneratorSelection;

GeneratorSelection SelectInputGenerator(Settings & s);
void MainRBF(Settings & s, GNGGLUTRepresentation & glutRep, SGNGAlgorithm * alg);
void MainGNG(Settings & s, GNGGLUTRepresentation & glutRep, GNGAlgorithm * alg);

int main(int argc, char** argv)
{
	GNGGLUTRepresentation glutRep(argv, argc);

	Settings s("gng.ini");

	try
	{     	
		if( s.m_mode == "sgng" )
			MainRBF(s, glutRep, new SGNGAlgorithm(s.m_GNGMSEBackLog));
		else
		{
			GNGAlgorithm * alg = 0;

		    if( s.m_mode == "gng" )
				alg = new GNGAlgorithm(s.m_GNGMSEBackLog);
			else
		    if( s.m_mode == "gngu" )			
			{
				GNGUAlgorithm * a = new GNGUAlgorithm(s.m_GNGMSEBackLog);
				a->SetParamUtility( s.m_utilityBound );        // GNG-U
				a->SetParamUtilityDecay( s.m_utilityDecay );   // GNG-U
				alg = a;
			}
			else
			  throw Exception("unrecognised mode. should be one of (gng, gngu, sgng)");
			  
			MainGNG(s, glutRep, alg);
		}
	}
	catch(Exception e)
	{
		cerr << "Exception:: " << endl;
		cerr << e.ToString() << endl << "Ending. press <enter>" << endl;
		cin.get();		
	}
	return 0;
}

//*******************************************
//*******************************************
//*******************************************

void MainGNG(Settings & s, GNGGLUTRepresentation & glutRep, GNGAlgorithm * alg)
{		
	GeneratorSelection selection = SelectInputGenerator(s);
	IInputGenerator * inputGenerator = selection.first;
	IGLInputRepresentation * inputRep = selection.second;    

	unsigned int dimension = inputGenerator->GetDimension();

	DefaultNodeInserter   nodeInserter;
	DefaultNodeMovement   nodeMovement;
	DefaultGNGContainer   gngContainer;

	gngContainer.SetNodeFactory( new GNGNodeFactory() );    
	nodeInserter.SetParamInsertEvery( s.m_insertEvery );
	nodeInserter.SetParamMaxNodes( s.m_maxNodes );
	nodeInserter.SetParamNewNodePos( s.m_newNodePos );
	nodeInserter.SetParamErrorDecay( s.m_errorDecayInsert );
	nodeInserter.SetGNGContainer( &gngContainer );

	alg->SetComponentGNGContainer( &gngContainer );
	alg->SetComponentNodeInserter( &nodeInserter );
	alg->SetComponentWinnerMovement( &nodeMovement );
	alg->SetComponentNeighborMovement( &nodeMovement );

	alg->SetParamAgeMax( s.m_ageMax );
	alg->SetParamMoveWinner( s.m_moveWinner );
	alg->SetParamMoveNeighbor( s.m_moveNeighbor );
	alg->SetParamErrorDecay( s.m_errorDecay );

	ofstream f("log_GNGMSE.txt");
	alg->SetMSEOutputStream(&f);

	if(s.m_initFromDistribution)
	  alg->Initialize(dimension, inputGenerator);	
    else
	  alg->Initialize(dimension);	

	GNGControl control(alg, inputGenerator);
	control.SetDelay( s.m_iterationDelay );
	
	glutRep.SetGNGContainer( &gngContainer );
	glutRep.SetInputRepresentation( inputRep );
	glutRep.SetControlObject( &control );
	glutRep.SetUpdateInterval( s.m_representationUpdate );        
	glutRep.Run();

	f.close();
}

//*******************************************
//*******************************************
//*******************************************


void MainRBF(Settings & s, GNGGLUTRepresentation & glutRep, SGNGAlgorithm * alg)
{	
	GeneratorSelection selection = SelectInputGenerator(s);
	IInputGenerator * inputGenerator = selection.first;
	IGLInputRepresentation * inputRep = selection.second;    
	
    unsigned int RBFnumOutputs = inputGenerator->GetOutput().size();
	unsigned int dimension = inputGenerator->GetDimension();

	// RBF stuff
	LinearActivation  activation;
	RBFWeightLayer    weightLayer(RBFnumOutputs);
	RBFHiddenLayer    hiddenLayer(&weightLayer);
	RBFOutputLayer    outputLayer(&activation, RBFnumOutputs, &weightLayer);    

	// GNG stuff
 	//DefaultNodeInserter   nodeInserter;
 	SGNGNodeInserter      nodeInserter;
	DefaultNodeMovement   nodeMovement;
	IGNGContainer *       gngContainer = &hiddenLayer;

	gngContainer->SetNodeFactory( new RBFNodeFactory( 2.0 ) );	
	nodeInserter.SetParamInsertEvery( s.m_insertEvery );
	nodeInserter.SetParamMaxNodes( s.m_maxNodes );
	nodeInserter.SetParamNewNodePos( s.m_newNodePos );
	nodeInserter.SetParamErrorDecay( s.m_errorDecayInsert );
	nodeInserter.SetGNGContainer( &hiddenLayer );

	alg->SetComponentGNGContainer( gngContainer );
	alg->SetComponentNodeInserter( &nodeInserter );
	alg->SetComponentWinnerMovement( &nodeMovement );
	alg->SetComponentNeighborMovement( &nodeMovement );

	alg->SetParamAgeMax( s.m_ageMax );
	alg->SetParamMoveWinner( s.m_moveWinner );
	alg->SetParamMoveNeighbor( s.m_moveNeighbor );
	alg->SetParamErrorDecay( s.m_errorDecay );

	ofstream f("log_GNGMSE.txt");
	alg->SetMSEOutputStream(&f);

	if(s.m_initFromDistribution)
	  alg->Initialize(dimension, inputGenerator);	
    else
	  alg->Initialize(dimension);	

	RBFNetwork net(&hiddenLayer, &weightLayer, &outputLayer);
	net.SetParameterEta( s.m_eta );

	Control * control = 0;
	if(inputGenerator->GetGeneratorType() == IInputGenerator::ClassesGenerator)
	{
	  // use as classifier
      ofstream * log = new ofstream("log_Classification.txt");
	  control = new RBFClassificationControl(net, inputGenerator, alg, 
                                             s.m_maxMisclassification, 
                                             s.m_RBFbackLog, 
                                             s.m_classificaitonDecisionLimit,
                                             s.m_classificaitonDecisionDelta,
                                             log);
    }
	else
	{
	  // use as classifier
      ofstream * log = new ofstream("log_RBFError.txt");
	  control = new RBFFunctionApproximationControl(net, inputGenerator, alg, 
                                                    s.m_maxSquaredError, 
                                                    s.m_RBFbackLog, 
                                                    log);
    }

	control->SetDelay( s.m_iterationDelay );

	glutRep.SetGNGContainer( gngContainer );
	glutRep.SetInputRepresentation( inputRep );
	glutRep.SetControlObject( control );
	glutRep.SetUpdateInterval( s.m_representationUpdate );
	glutRep.Run();

	f.close();

}



//-------------------------------
//  Different combinations of InputGenerators
//-------------------------------
GeneratorSelection SelectInputGenerator(Settings & s)
{
	IInputGenerator * inputGenerator = 0;
	IGLInputRepresentation * inputRep = 0;

	switch(s.m_distributionType)
	{
	    // Default
	case 0:
		{
			InputGeneratorDefault * ig = new InputGeneratorDefault( s.m_dim );
			inputGenerator = ig;
			inputRep = new InputRepresentationDefault(ig);    
		}
		break;
	
		// Four of default
	case 1:
		{
			InputGeneratorDefault * i = new InputGeneratorDefault( 2 );
			InputGeneratorFour * ig = new InputGeneratorFour(i, 0.30);  // make four of them
			inputGenerator = ig;

			InputRepresentationDefault * ir = new InputRepresentationDefault(i);    
			inputRep = new InputRepresentationFour( ig, ir );
		}
		break;

		// Jump Default
	case 2:
		{
			InputGeneratorDefault * i = new InputGeneratorDefault(2);      // a square
			InputGeneratorJump * ig = new InputGeneratorJump(i, 20000, 0.35);
			inputGenerator = ig;

			InputRepresentationDefault * ir = new InputRepresentationDefault( i );    
			inputRep = new InputRepresentationJump( ig, ir );
		}
		break;

		// Jump Square moving default
	case 3:
		{
			InputGeneratorDefault * i = new InputGeneratorDefault(2);      // a square
			InputGeneratorDefaultNonstationary * i2 = new InputGeneratorDefaultNonstationary(i, 0.0002, 0.35);  // make them move
			InputGeneratorJump * ig = new InputGeneratorJump(i2, 25000, 0.75);
			inputGenerator = ig;

			InputRepresentationDefault * ir = new InputRepresentationDefault( i );    
			InputRepresentationDefaultNonstationary * ir2 = new InputRepresentationDefaultNonstationary( i2, ir );
			inputRep = new InputRepresentationJump( ig, ir2 );
		}
		break;

		// Square slow Moving Default
	case 4:
		{
			InputGeneratorDefault * i = new InputGeneratorDefault(2);      // a square
			InputGeneratorDefaultNonstationary * ig = new InputGeneratorDefaultNonstationary(i, 0.000008, 0.25);  // make them move
			inputGenerator = ig;

			InputRepresentationDefault * ir = new InputRepresentationDefault( i );    
			inputRep = new InputRepresentationDefaultNonstationary( ig, ir );
		}
		break;


		// Square Moving Four of Default
	case 5:
		{
			InputGeneratorDefault * i = new InputGeneratorDefault(2);      // a square
			InputGeneratorFour * i2 = new InputGeneratorFour(i, 0.30);  // make four of them
			InputGeneratorDefaultNonstationary * ig = new InputGeneratorDefaultNonstationary(i2, 0.0001, 0.25);  // make them move
			inputGenerator = ig;

			InputRepresentationDefault * ir = new InputRepresentationDefault( i );    
			InputRepresentationFour * ir2 = new InputRepresentationFour( i2, ir );
			inputRep = new InputRepresentationDefaultNonstationary( ig, ir2 );
		}
		break;

		// Four of Square Moving Default
	case 6:
		{
			InputGeneratorDefault * i = new InputGeneratorDefault(2);      // a square
			InputGeneratorDefaultNonstationary * i2 = new InputGeneratorDefaultNonstationary(i, 0.0001, 0.25);  // make them move
			InputGeneratorFour * ig = new InputGeneratorFour(i2, 0.30);  // make four of them
			inputGenerator = ig;

			InputRepresentationDefault * ir = new InputRepresentationDefault( i );    
			InputRepresentationDefaultNonstationary * ir2 = new InputRepresentationDefaultNonstationary( i2, ir );
			inputRep = new InputRepresentationFour( ig, ir2 );
		}
		break;

		// Four of Four of Default
	case 7:
		{
			InputGeneratorDefault * i = new InputGeneratorDefault(2);      // a square
			InputGeneratorFour * i2 = new InputGeneratorFour(i, 0.40);     // make four of them
			InputGeneratorFour * ig = new InputGeneratorFour(i2, 0.40);    // make 16 of them
			inputGenerator = ig;

			InputRepresentationDefault * ir = new InputRepresentationDefault( i );
			InputRepresentationFour * ir2 = new InputRepresentationFour( i2, ir );
			inputRep = new InputRepresentationFour( ig, ir2 );
		}
		break;

		// UpDown Moving of Default
	case 8:
		{
			InputGeneratorDefault * i = new InputGeneratorDefault(2);      // a square
			InputGeneratorUpDown * ig = new InputGeneratorUpDown(i, 0.0001, 0.25);  // make them move
			inputGenerator = ig;

			InputRepresentationDefault * ir = new InputRepresentationDefault( i );    
			inputRep = new InputRepresentationUpDown( ig, ir );
		}
		break;

		// UpDown Moving, Square Moving of Default
	case 9:
		{
			InputGeneratorDefault *i = new InputGeneratorDefault(2);      // a square
			InputGeneratorDefaultNonstationary * i2 = new InputGeneratorDefaultNonstationary(i, 0.0002, 0.25);  // make them move
			InputGeneratorUpDown * ig = new InputGeneratorUpDown(i2, 0.0001, 0.7);  // make them move
			inputGenerator = ig;

			InputRepresentationDefault * ir = new InputRepresentationDefault( i );    
			InputRepresentationDefaultNonstationary * ir2 = new InputRepresentationDefaultNonstationary( i2, ir );
			inputRep = new InputRepresentationUpDown( ig, ir2 );
		}
		break;

		// Square Moving, UpDown Moving, of Default
	case 10:
		{
			InputGeneratorDefault * i = new InputGeneratorDefault(2);      // a square
			InputGeneratorUpDown * i2 = new InputGeneratorUpDown(i, 0.0002, 0.40);  // make them move
			InputGeneratorDefaultNonstationary * ig = new InputGeneratorDefaultNonstationary(i2, 0.0001, 0.25);  // make them move
			inputGenerator = ig;

			InputRepresentationDefault * ir = new InputRepresentationDefault( i );    
			InputRepresentationUpDown * ir2 = new InputRepresentationUpDown( i2, ir );
			inputRep = new InputRepresentationDefaultNonstationary( ig, ir2);
		}
		break;
		
		// TwoClasses
    case 11:
        {
	        InputGeneratorTwoClasses * ig = new InputGeneratorTwoClasses(0.8, 0.5);
	        inputGenerator = ig;
	        inputRep = new InputRepresentationTwoClasses(ig);
        }
        break;

		// XOR
    case 12:
        {            
	        InputGeneratorXOR * ig = new InputGeneratorXOR;
	        inputGenerator = ig;
		    inputRep = new InputRepresentationXOR(ig);
        }
        break;

		// Discrete
    case 13:
        {
            RawReader r("discrete_100x100_8bit.raw", 100, 100);
	        InputGeneratorDiscrete * ig = new InputGeneratorDiscrete(r.Convert());
	        inputGenerator = ig;
            inputRep = new InputRepresentationDiscrete(ig);
        }
        break;
        
        // Sinus    
    case 14:
        {
            InputGeneratorSinus * ig = new InputGeneratorSinus(100, 0.0, 2.0*InputGeneratorSinus::PI);
            inputGenerator = ig;
            inputRep = new InputRepresentationSinus(ig);
        }
        break;

	default:
		{
			InputGeneratorDefault * ig = new InputGeneratorDefault( s.m_dim );
			inputGenerator = ig;
			inputRep = new InputRepresentationDefault(ig);    
		}
		break;
	}

	GeneratorSelection selection;
	selection.first = inputGenerator;
	selection.second = inputRep;
	return selection;	
} 
